/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.dtaus.swing;

import java.awt.Component;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.EmptyStackException;
import java.util.Stack;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import org.jdtaus.banking.swing.BankleitzahlenVerzeichnisDialog;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.ContainerInitializer;
import org.jdtaus.core.container.Dependency;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.Implementations;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;
import org.jdtaus.editor.PhysicalFileFormatter;

/**
 * GUI of the client application.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: EditorGUI.java 3429 2007-05-11 05:21:27Z schulte2005 $
 */
public class EditorGUI implements ContainerInitializer
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(EditorGUI.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Protected <code>EditorGUI</code> implementation constructor.
     *
     * @param meta Implementation meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected EditorGUI(final Implementation meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }
    /**
     * Protected <code>EditorGUI</code> dependency constructor.
     *
     * @param meta dependency meta-data.
     *
     * @throws NullPointerException if <code>meta</code> is <code>null</code>.
     */
    protected EditorGUI(final Dependency meta)
    {
        super();
        if(meta == null)
        {
            throw new NullPointerException("meta");
        }
        this.initializeProperties(meta.getProperties());
    }

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------

    public void initialize()
    {
        this.main = new PhysicalFileFrame();
        this.main.addWindowListener(new WindowListener()
        {
            public void windowOpened(final WindowEvent e)
            {
                main.setExtendedState(JFrame.MAXIMIZED_BOTH);
            }

            public void windowClosing(final WindowEvent e)
            {
            }

            public void windowClosed(final WindowEvent e)
            {
                System.exit(0);
            }

            public void windowIconified(final WindowEvent e)
            {
            }

            public void windowDeiconified(final WindowEvent e)
            {
            }

            public void windowActivated(final WindowEvent e)
            {
            }

            public void windowDeactivated(final WindowEvent e)
            {
            }
        });

        this.push(this.main);
    }

    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--EditorGUI---------------------------------------------------------------

    /**
     * Holds the singleton {@code PhysicalFileFrame} instance.
     * @serial
     */
    private PhysicalFileFrame main;

    /**
     * Stack holding parent components.
     * @serial
     */
    private Stack components = new Stack();

    /**
     * Flag indicating that the user interface is locked.
     * @serial
     */
    private boolean locked;

    /**
     * Dialog to display and search the {@code BankleitzahlenVerzeichnis}.
     * @serial
     */
    private BankleitzahlenVerzeichnisDialog blzDialog;

    /**
     * Dialog to display and edit a {@code Header}.
     * @serial
     */
    private HeaderDialog headerDialog;

    /**
     * Dialog to display and edit a {@code Transaction}.
     * @serial
     */
    private TransactionDialog transactionDialog;

    /**
     * Dialog for generating DTAUS files.
     * @serial
     */
    private GeneratorDialog generatorDialog;

    /**
     * Dialog for application configuration.
     * @serial
     */
    private ConfigurationDialog configurationDialog;

    /**
     * Filechooser for opening.
     * @serial
     */
    private JFileChooser openFileChooser;

    /**
     * Filechooser for Save as.
     * @serial
     */
    private JFileChooser saveAsFileChooser;

    /**
     * Gets the current parent component of the user interface.
     *
     * @return the current parent component of the user interface.
     *
     * @throws EmptyStackException if the parent component stack is empty.
     */
    public Component getParent() throws EmptyStackException
    {
        return (Component) this.components.peek();
    }

    /**
     * Pushes a component onto the stack of parent components.
     *
     * @param component  the current parent component.
     *
     * @throws NullPointerException if {@code component} is {@code null}.
     */
    public void push(final Component component)
    {
        if(component == null)
        {
            throw new NullPointerException("component");
        }

        this.components.push(component);
    }

    /**
     * Pops the current parent component off the stack of parent components.
     *
     * @return the current parent component popped off the stack.
     *
     * @throws EmptyStackException if the parent component stack is empty.
     */
    public Component pop()
    {
        return (Component) this.components.pop();
    }

    /** Requests an update of all user interface components. */
    public void updateComponentTreeUI()
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                SwingUtilities.updateComponentTreeUI(
                    getBankleitzahlenVerzeichnisDialog());

                SwingUtilities.updateComponentTreeUI(getConfigurationDialog());
                SwingUtilities.updateComponentTreeUI(getOpenFileChooser());
                SwingUtilities.updateComponentTreeUI(getSaveAsFileChooser());
                SwingUtilities.updateComponentTreeUI(getGeneratorDialog());
                SwingUtilities.updateComponentTreeUI(getHeaderDialog());
                SwingUtilities.updateComponentTreeUI(getTransactionDialog());
                SwingUtilities.updateComponentTreeUI(EditorGUI.this.main);

                EditorGUI.this.main.pack();
                EditorGUI.this.main.setExtendedState(JFrame.MAXIMIZED_BOTH);

                getConfigurationDialog().pack();
                getGeneratorDialog().pack();
                getHeaderDialog().pack();
                getTransactionDialog().pack();
            }
        });
    }

    /**
     * Flag indicating if the user interface is locked for any interaction.
     *
     * @return {@code true} if the user interface is locked for any interaction;
     * {@code false} if the user interface is not locked.
     */
    public boolean isLocked()
    {
        return this.locked;
    }

    /**
     * Locks or unlocks the user interface.
     *
     * @param locked {@code true} to lock the user interface for any
     * interaction; {@code false} to unlock the user interface.
     */
    public void setLocked(final boolean locked)
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                main.setLocked(locked);
                getBankleitzahlenVerzeichnisDialog().setLocked(locked);
                getHeaderDialog().setLocked(locked);
                getTransactionDialog().setLocked(locked);
                getGeneratorDialog().setLocked(locked);
                getConfigurationDialog().setLocked(locked);
            }
        });
        this.locked = locked;
    }

    public BankleitzahlenVerzeichnisDialog getBankleitzahlenVerzeichnisDialog()
    {
        if(this.blzDialog == null)
        {
            this.blzDialog =
                new BankleitzahlenVerzeichnisDialog(this.main, true);

        }

        return this.blzDialog;
    }

    public HeaderDialog getHeaderDialog()
    {
        if(this.headerDialog == null)
        {
            this.headerDialog = new HeaderDialog(this.main, true);
        }

        return this.headerDialog;
    }

    public TransactionDialog getTransactionDialog()
    {
        if(this.transactionDialog == null)
        {
            this.transactionDialog =
                new TransactionDialog(this.main, true);
        }

        return this.transactionDialog;
    }

    public GeneratorDialog getGeneratorDialog()
    {
        if(this.generatorDialog == null)
        {
            this.generatorDialog =
                new GeneratorDialog(this.main, true);
        }

        return this.generatorDialog;
    }

    public ConfigurationDialog getConfigurationDialog()
    {
        if(this.configurationDialog == null)
        {
            this.configurationDialog =
                new ConfigurationDialog(this.main, true);
        }

        return this.configurationDialog;
    }

    public JFileChooser getOpenFileChooser()
    {
        if(this.openFileChooser == null)
        {
            this.openFileChooser = new JFileChooser();
        }

        return this.openFileChooser;
    }

    public JFileChooser getSaveAsFileChooser()
    {
        if(this.saveAsFileChooser == null)
        {
            this.saveAsFileChooser = new JFileChooser();
            this.saveAsFileChooser.setAcceptAllFileFilterUsed(false);
            this.saveAsFileChooser.setMultiSelectionEnabled(false);

            final Implementations formatters =
                ModelFactory.getModel().getModules().
                getSpecification(PhysicalFileFormatter.class.getName()).
                getImplementations();

            for(int i = formatters.size() - 1; i >= 0; i--)
            {
                final PhysicalFileFormatter formatter =
                    (PhysicalFileFormatter) ContainerFactory.
                    getContainer().getImplementation(
                    PhysicalFileFormatter.class,
                    formatters.getImplementation(i).getName());

                this.saveAsFileChooser.addChoosableFileFilter(
                    new PhysicalFileFormatterFileFilter(formatter));

            }
        }

        return this.saveAsFileChooser;
    }

    //---------------------------------------------------------------EditorGUI--

}
