/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.banking.swing;

import java.text.ParseException;
import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import org.jdtaus.banking.Referenznummer10;
import org.jdtaus.core.container.Implementation;
import org.jdtaus.core.container.ModelFactory;
import org.jdtaus.core.container.Properties;
import org.jdtaus.core.container.Property;

/**
 * Extension to {@code JFormattedTextField} adding support for the
 * {@link Referenznummer10} type.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: Referenznummer10TextField.java 3429 2007-05-11 05:21:27Z schulte2005 $
 */
public final class Referenznummer10TextField extends JFormattedTextField
{
    //--Implementation----------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /** Meta-data describing the implementation. */
    private static final Implementation META =
        ModelFactory.getModel().getModules().
        getImplementation(Referenznummer10TextField.class.getName());

    //----------------------------------------------------------Implementation--
    //--Constructors------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.

    /**
     * Initializes the properties of the instance.
     *
     * @param meta the property values to initialize the instance with.
     *
     * @throws NullPointerException if {@code meta} is {@code null}.
     */
    protected void initializeProperties(final Properties meta)
    {
        Property p;

        if(meta == null)
        {
            throw new NullPointerException("meta");
        }

    }

    //------------------------------------------------------------Constructors--
    //--ContainerInitializer----------------------------------------------------
    //----------------------------------------------------ContainerInitializer--
    //--Dependencies------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //------------------------------------------------------------Dependencies--
    //--Properties--------------------------------------------------------------

    // This section is managed by jdtaus-container-mojo.


    //--------------------------------------------------------------Properties--
    //--Referenznummer10TextField-----------------------------------------------

    /** Creates a new {@code Referenznummer10TextField} instance. */
    public Referenznummer10TextField()
    {
        super(new JFormattedTextField.AbstractFormatter()
        {
            public Object stringToValue(final String text)
            throws ParseException
            {
                return Referenznummer10.parse(text);
            }

            public String valueToString(final Object value)
            throws ParseException
            {
                String ret = null;

                if(value instanceof Referenznummer10)
                {
                    ret = ((Referenznummer10) value).
                        format(Referenznummer10.ELECTRONIC_FORMAT);

                }

                return ret;
            }
        });

        /*
        this.setInputVerifier(new InputVerifier()
        {
            public boolean verify(final JComponent input)
            {
                boolean valid = true;

                if(input instanceof Referenznummer10TextField)
                {
                    try
                    {
                        Referenznummer10.parse(
                            ((Referenznummer10TextField) input).getText());

                    }
                    catch(ParseException e)
                    {
                        valid = false;
                    }
                }

                return valid;
            }
        });
         */
    }

    /**
     * Gets the last valid value.
     *
     * @return the last valid value or {@code null}.
     */
    public Referenznummer10 getReferenznummer10()
    {
        return (Referenznummer10) super.getValue();
    }

    //-----------------------------------------------Referenznummer10TextField--

}
