/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.jdtaus.editor;

import java.io.Serializable;

/**
 * User preferences.
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: Preferences.java 3542 2007-05-21 00:19:35Z schulte2005 $
 */
public class Preferences implements Serializable
{
    //--Preferences-------------------------------------------------------------

    /**
     * Flag indicating if bankcode validation is disabled.
     * @serial
     */
    private boolean bankcodeValidationDisabled;

    /**
     * Flag indicating if confirmation of A record deletions is disabled.
     * @serial
     */
    private boolean fileDeletionConfirmationDisabled;

    /**
     * Flag indicating if confirmation of C record deletions is disabled.
     * @serial
     */
    private boolean paymentDeletionConfirmationDisabled;

    /**
     * Flag indicating if confirmation for overwriting files is disabled.
     * @serial
     */
    private boolean overwriteConfirmationDisabled;

    /**
     * Gets a flag indicating if bankcode validation is disabled.
     *
     * @return {@code true} if bankcode validation is disabled; {@code false}
     * if not.
     */
    public boolean isBankcodeValidationDisabled()
    {
        return this.bankcodeValidationDisabled;
    }

    /**
     * Sets the flag indicating if bankcode validation is disabled.
     *
     * @param value {@code true} if bankcode validation should be disabled;
     * {@code false} if not.
     */
    public void setBankcodeValidationDisabled(final boolean value)
    {
        this.bankcodeValidationDisabled = value;
    }

    /**
     * Gets a flag indicating if confirmation of logical file deletions is
     * disabled.
     *
     * @return {@code true} if confirmation of logical file deletions is
     * disabled; {@code false} if not.
     */
    public boolean isFileDeletionConfirmationDisabled()
    {
        return this.fileDeletionConfirmationDisabled;
    }

    /**
     * Sets the flag indicating if confirmation of logical file deletions is
     * disabled.
     *
     * @param value {@code true} if confirmation of logical file deletions
     * should  be disabled; {@code false} if not.
     */
    public void setFileDeletionConfirmationDisabled(final boolean value)
    {
        this.fileDeletionConfirmationDisabled = value;
    }

    /**
     * Gets a flag indicating if confirmation of payment deletions is disabled.
     *
     * @return {@code true} if confirmation of payment deletions is disabled;
     * {@code false} if not.
     */
    public boolean isPaymentDeletionConfirmationDisabled()
    {
        return this.paymentDeletionConfirmationDisabled;
    }

    /**
     * Sets the flag indicating if confirmation of payment deletions is
     * disabled.
     *
     * @param value {@code true} if confirmation of payment deletions should be
     * disabled; {@code false} if not.
     */
    public void setPaymentDeletionConfirmationDisabled(final boolean value)
    {
        this.paymentDeletionConfirmationDisabled = value;
    }

    /**
     * Gets a flag indicating if confirmation for overwriting files is disabled.
     *
     * @return {@code true} if confirmation for overwriting is disabled;
     * {@code false} if not.
     */
    public boolean isOverwriteConfirmationDisabled()
    {
        return this.overwriteConfirmationDisabled;
    }

    /**
     * Sets the flag indicating if confirmation for overwriting files is
     * disabled.
     *
     * @param value {@code true} if confirmation for overwriting files should be
     * disabled; {@code false} if not.
     */
    public void setOverwriteConfirmationDisabled(final boolean value)
    {
        this.overwriteConfirmationDisabled = value;
    }

    /**
     * Creates a string representing the properties of the instance.
     *
     * @return a string representing the properties of the instance.
     */
    private String internalString()
    {
        return new StringBuffer(150).
            append("\n\tbankcodeValidationDisabled=").
            append(this.bankcodeValidationDisabled).

            append("\n\tfileDeletionConfirmationDisabled=").
            append(this.fileDeletionConfirmationDisabled).

            append("\n\toverwriteConfirmationDisabled=").
            append(this.overwriteConfirmationDisabled).

            append("\n\tpaymentDeletionConfirmationDisabled=").
            append(this.paymentDeletionConfirmationDisabled).

            toString();

    }

    //-------------------------------------------------------------Preferences--
    //--Object------------------------------------------------------------------

    /**
     * Returns a string representation of the object.
     *
     * @return a string representation of the object.
     */
    public String toString()
    {
        return super.toString() + this.internalString();
    }

    //------------------------------------------------------------------Object--
}
